package com.catkissfish.springboot.mocikito.demo;

import com.catkissfish.springboot.mocikito.demo.dto.UserUpdateReqDto;
import com.catkissfish.springboot.mocikito.demo.entity.User;
import com.catkissfish.springboot.mocikito.demo.service.UserService;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentMatchers;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;

/**
 * 参数匹配测试：通过方法签名(参数)来指定哪些方法调用需要被处理(插桩、verify验证)
 */
@ExtendWith(MockitoExtension.class)
public class Demo02 {
    @Mock
    private UserService mockUserService;

    /**
     * mock对象没有插桩，不会调用真实方法，直接返回mock对象的默认值
     * 如(int)默认值就是0，对象的默认值就是null，空集合的默认值就是长度为0的集合。
     */
    @Test
    public void test1(){
        User user = mockUserService.selectOne("zs");
        System.out.println("user = " + user);
        UserUpdateReqDto reqDto = new UserUpdateReqDto();
        int i = mockUserService.modifyById(reqDto);
        System.out.println("i = " + i);
    }

    /**
     * 测试插桩时的参数匹配,只拦截userUpdateReqDto
     */
    @Test
    public void test2(){
        UserUpdateReqDto userUpdateReqDto = new UserUpdateReqDto();
        userUpdateReqDto.setId(1L);
        userUpdateReqDto.setUsername("zs");
        //指定参数为userUpdateReqDto时调用mockUserService.modifyById返回99
        Mockito.doReturn(99).when(mockUserService).modifyById(userUpdateReqDto);
        int result1 = mockUserService.modifyById(userUpdateReqDto);
        System.out.println("result1 = " + result1);

        UserUpdateReqDto userUpdateReqDto2 = new UserUpdateReqDto();
        userUpdateReqDto2.setId(2L);
        userUpdateReqDto2.setUsername("ls");
        int result2 = mockUserService.modifyById(userUpdateReqDto2);
        System.out.println("result2 = " + result2);
    }

    /**
     * ArgumentMatchers.any拦截UserUpdateReqDto类型的任意对象
     */
    @Test
    public void test3(){
        Mockito.doReturn(99).when(mockUserService).modifyById(ArgumentMatchers.any(UserUpdateReqDto.class));
        UserUpdateReqDto userUpdateReqDto = new UserUpdateReqDto();
        userUpdateReqDto.setId(1L);
        userUpdateReqDto.setUsername("zs");
        int result1 = mockUserService.modifyById(userUpdateReqDto);
        System.out.println("result1 = " + result1);

        UserUpdateReqDto userUpdateReqDto2 = new UserUpdateReqDto();
        userUpdateReqDto2.setId(2L);
        userUpdateReqDto2.setUsername("ls");
        int result2 = mockUserService.modifyById(userUpdateReqDto2);
        System.out.println("result2 = " + result2);
    }

    /**
     *
     */
    @Test
    public void test4(){
        User user = new User();
        user.setName("zs");
        user.setPassword("123456");
        mockUserService.add(user);
//        mockUserService.add(user);
        //校验调用了add方法一次
        Mockito.verify(mockUserService, Mockito.times(1)).add(user);
    }

}
